package at.htl.leonding.swimresults.editswimmer

import android.app.Application
import android.util.Log
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import at.htl.leonding.swimresults.database.SwimResultsDao
import at.htl.leonding.swimresults.database.Swimmer
import kotlinx.coroutines.*
import org.jsoup.Jsoup
import java.net.URL

class EditSwimmerViewModel(
        private val swimmerId: Long = 0L,
        val database: SwimResultsDao, application: Application) : AndroidViewModel(application) {

    companion object{
        val LOG_TAG = EditSwimmerViewModel::class.java.simpleName
    }

    private val viewModelJob = Job()
    private val uiScope = CoroutineScope(Dispatchers.Main + viewModelJob)

    val editSwimmerId = MutableLiveData<String>()
    private var swimmer = MutableLiveData<Swimmer?>()

    init{
        uiScope.launch{
            swimmer.value = getSwimmer(swimmerId)
            swimmer.value?.let{
                editSwimmerId.value = it.swimmerId.toString()
            }
        }
    }

    private var _navigateBack = MutableLiveData<Boolean>()
    val navigateBack: LiveData<Boolean?>
        get () = _navigateBack
    fun doneNavigating(){
        _navigateBack.value = null
    }

    var swimmerTitle = Transformations.map(swimmer){s ->
        if (s != null) {
            "${s.firstName} ${s.lastName} (${s.ageGroup})"
        }
        else{
            "Keine Daten vorhanden"
        }
    }

    var swimmerClub = Transformations.map(swimmer){s ->
        if (s != null) {
            "${s.club}"
        }
        else{
            ""
        }
    }

    val statusString = Transformations.map(editSwimmerId){ osvId ->
        Log.d(LOG_TAG, "OSV ID updated")
        if (osvId == null || (osvId.length != 6 && osvId.length != 5)){
            swimmer.value = null
            "ID muss 5 oder 6 Stellen lang sein"
        }
        else{
            uiScope.launch{
                swimmer.value = loadFromNetwork(osvId)
            }
            ""
        }
    }

    fun onSave(){
        uiScope.launch{
            swimmer.value?.let{
                val dbSwimmer = getSwimmer(it.swimmerId)
                when{
                    (dbSwimmer == null) -> insert(it)
                    (dbSwimmer != it) -> update(it)
                }
            }
            _navigateBack.value = true
        }
    }

    fun onCancel(){
        _navigateBack.value = true
    }

    suspend fun loadFromNetwork(osvId: String) : Swimmer?{
        return withContext(Dispatchers.IO){
            val urlString = "https://ergebnisse.osv.or.at/modules/Results/Stroke.aspx?SwimmerID=${osvId}"
            // val urlString = "https://ergebnisse.osv.or.at/modules/Results/Best.aspx?SwimmerID=${editSwimmerId}&Lang=de-DE"
            val url = URL(urlString)
            Log.d(LOG_TAG, "Loading URL ${url}")
            var loadedSwimmer: Swimmer?
            Jsoup.connect(urlString).get().run {
                val names = select("span[id=_nameLabel]").text().split(' ')
                if (names.size != 2){
                    loadedSwimmer = null
                    return@run
                }
                val firstName = names[0]
                val lastName = names[1]
                val ageGroup = select("span[id=_bornDataLabel]").text().split(' ')[0]
                val clubString = select("span[id=_clubDataLabel]")
                        .elementAt(0)
                        .toString()
                        .split(">")[1]
                val club = clubString.substring(0, clubString.indexOf('<'))

                val isMale = select("span[id=_genderDataLabel]").text().startsWith("m")
                loadedSwimmer = Swimmer(osvId.toLong(), firstName, lastName, ageGroup, club, isMale)
                Log.d(LOG_TAG, "Swimmer parsed: ${loadedSwimmer}")
            }
            loadedSwimmer
        }
    }

    suspend fun getSwimmer(key: Long) : Swimmer?{
        return withContext(Dispatchers.IO){
            return@withContext database.getSwimmer(key)
        }
    }

    suspend fun update(swimmer: Swimmer) {
        withContext(Dispatchers.IO){
            database.updateSwimmer(swimmer)
        }
    }

    suspend fun insert(swimmer: Swimmer) {
        withContext(Dispatchers.IO){
            database.insertSwimmer(swimmer)
        }
    }

    override fun onCleared() {
        super.onCleared()
        viewModelJob.cancel()
    }
}

